import axios from 'axios'
import router from '@/router/index'
import { Message, MessageBox } from 'element-ui'
import Tips from '../components/RequestTips/index'

/**
 * 提示函数
 * 禁止点击蒙层、显示一秒后关闭
 */
const tip = msg => {
  Message({
    message: '<div style="line-height: 160%">' + msg + '</div>',
    type: 'error',
    dangerouslyUseHTMLString: true,
    duration: 2000
  })
}
// create an axios instance
const service = axios.create({
  baseURL: process.env.VUE_APP_BASE_API, // url = base url + request url 跨域路径  /mock 模拟数据
  timeout: 90000 // request timeout
})

service.defaults.headers.common['Cache-Control'] = 'no-cache'
/**
 * 请求拦截器
 */
service.interceptors.request.use(
  config => {
    config.headers['X-Requested-Host'] = (window.location.hostname || window.location.host).replace(/:(\d+)$/, '')
    if (config.requestedHost) {
      config.headers['X-Requested-Host'] = config.requestedHost
    }
    const method = config.method.toUpperCase()
    if (
      method === 'POST' ||
      method === 'PUT' ||
      method === 'DELETE' ||
      typeof config.processMessage !== 'undefined'
    ) {
      if (!config.noDialog) {
        config.$$dialog = Message({
          message: config.processMessage || '正在执行操作...',
          type: 'info',
          duration: 0
        })
      }
    } else {
      if (!config.noDialog) {
        config.$$dialog = Tips.show('正在加载···')
      }
    }
    if (config.ignoreInterceptors === true) return config
    return config
  },
  error => {
    if (error.config.$$dialog) error.config.$$dialog.close()
    return Promise.error(error)
  }
)
/**
 * 请求失败后的错误统一处理
 * @param {Number} status 请求失败的状态码
 */
const errorHandle = (status, other) => {
  let html = ''

  if (typeof other.errors !== 'undefined') {
    const data = other.errors
    for (const key in data) {
      if (data.hasOwnProperty(key)) {
        const element = data[key]
        html += element + '<br/>'
      }
    }
    MessageBox.alert(html, '系统提示', {
      dangerouslyUseHTMLString: true,
      type: 'error'
    })
    // 如果不存在errors 存在message
  } else if (typeof other.message !== 'undefined' && other.message !== null) {
    html = ''
    if (typeof other.message === 'object') {
      const messageArray = Object.keys(other.message)
      // 如果只有一个数组
      if (messageArray.length === 1) {
        const messageArrayFirst = other.message[messageArray[0]]
        if (messageArrayFirst instanceof Array) {
          html += messageArrayFirst.join('，')
        }
      } else {
        for (const key in other.message) {
          if (other.message.hasOwnProperty(key)) {
            const element = other.message[key]
            if (element instanceof Array) {
              html += element.join('，') + '<br/>'
            } else {
              html += element + '<br/>'
            }
          }
        }
      }
      tip(html)
    } else {
      tip(other.message)
    }
  }
  if (status === 401) {
    router.replace({
      name: 'AdminLogin',
      query: {
        redirect: router.currentRoute.fullPath
      }
    })
  }
}

// response interceptor
service.interceptors.response.use(
  response => {
    if (response.config.$$dialog) response.config.$$dialog.close()
    if (response.config.ignoreInterceptors === true) return response

    // 204或者data为空的，返回固定的格式...
    if (response.status === 204 || !response.data) {
      return {
        status_code: response.status,
        message: '操作成功！'
      }
    } else {
      return response.data
    }
  },

  error => {
    if (error.config.$$dialog) error.config.$$dialog.close()
    const { response } = error
    if (error.config.ignoreInterceptors === true) return Promise.reject(response)
    if (response && !error.config.noErrorTips) {
      // 请求已发出，但是不在2xx的范围
      errorHandle(response.status, response.data)
      return Promise.reject(response)
    } else {
      return Promise.reject(response)
    }
  }
)
export default service
