import axios from 'axios'
import { Toast, Dialog } from 'vant'
import router from '../router'

/**
 * 提示函数
 * 禁止点击蒙层、显示一秒后关闭
 */
const tip = (msg, useAlert, code = 0) => {
  // Notify({ type: 'warning', message: msg, duration: 2500 })
  if (code === 406) {
    Dialog.alert({
      title: '账户余额不足',
      message: msg,
      showCancelButton: true,
      cancelButtonText: '立即充值',
      beforeClose(action, done) {
        if (action === 'cancel') {
          window.location = '/user/payment'
          return
        }
        done()
      }
    })
    return
  }
  if (useAlert) {
    Dialog.alert({
      title: '操作失败',
      message: msg
    })
    return
  }
  Toast({
    message: msg,
    position: 'top',
    duration: 2500
  })
}

const service = axios.create({
  baseURL: process.env.VUE_APP_BASE_API, // url = base url + request url 跨域路径  /mock 模拟数据
  withCredentials: false, // send cookies when cross-domain requests
  timeout: 100000 // request timeout
})

// 设置post请求头
service.defaults.headers.post['Content-Type'] =
  'application/x-www-form-urlencoded'

/**
 * 请求拦截器
 */
service.interceptors.request.use(
  config => {
    const method = config.method.toUpperCase()
    if (
      config.showProcessMessage !== false &&
      (method === 'POST' ||
        method === 'PUT' ||
        method === 'DELETE' ||
        typeof config.processMessage !== 'undefined')
    ) {
      if (config.processMessage === '@loading') {
        config.$$dialog = Toast.loading({
          message: '',
          forbidClick: true,
          duration: 0
        })
      } else {
        config.$$dialog = Toast({
          message: config.processMessage || '正在执行操作...',
          position: 'top',
          duration: 0
        })
      }
    }
    return config
  },
  error => {
    if (error.config.$$dialog) error.config.$$dialog.clear()
    return Promise.error(error)
  }
)
/**
 * 请求失败后的错误统一处理
 * @param {Number} status 请求失败的状态码
 * @param {Object} other 请求失败的状态码
 * @param {Object} config 请求失败的状态码
 */
const errorHandle = (status, other, config) => {
  let html = ''
  // 是否禁止显示提示
  if (config.forbidShowMessage) {
    return
  }
  if (status !== 401 || config.showMessage !== false) {
    if (typeof other.errors !== 'undefined') {
      const data = other.errors
      for (const key in data) {
        if (data.hasOwnProperty(key)) {
          const element = data[key]
          html += element + '\n'
        }
      }
      tip(html, config.useAlert === true, status)
      // 如果不存在errors 存在message
    } else if (typeof other.message !== 'undefined' && other.message !== null) {
      if (typeof other.message === 'object') {
        html = ''
        for (const key in other.message) {
          if (other.message.hasOwnProperty(key)) {
            const element = other.message[key]
            html += element + '\n'
          }
        }
        tip(html, config.useAlert === true, status)
      } else {
        tip(other.message, config.useAlert === true, status)
      }
    }
  }
  if (status === 401) {
    if (config.redirect === false) return
    router.push({
      name: 'UserLogin',
      query: {
        redirect: router.app.$route.fullPath || ''
      }
    })
  }
}

// response interceptor
service.interceptors.response.use(
  response => {
    if (response.config.$$dialog) response.config.$$dialog.clear()

    return response.config.returnRaw === true ? response.data.data.data : response.data
  },
  error => {
    if (error.config.$$dialog) error.config.$$dialog.clear()
    const { response } = error
    if (response) {
      errorHandle(response.status, response.data, error.config)
      return Promise.reject(response.data)
    }
  }
)

export default service
