import axios from 'axios'
import { Message, MessageBox } from 'element-ui'
import store from '@/store'

/**
  * 提示函数
  * 禁止点击蒙层、显示一秒后关闭
  */
const tip = msg => {
  Message({
    message: msg,
    type: 'error',
    dangerouslyUseHTMLString: true,
    duration: 5 * 1000
  })
}
// create an axios instance
const service = axios.create({
  baseURL: process.env.VUE_APP_BASE_API, // url = base url + request url 跨域路径  /mock 模拟数据
  // withCredentials: true, // send cookies when cross-domain requests
  timeout: 90000 // request timeout
})

service.defaults.headers.common['Cache-Control'] = 'no-cache'
/**
  * 请求拦截器
  * 每次请求都在请求头中携带token
  */
service.interceptors.request.use(
  config => {
    return config
  },
  error => Promise.error(error)
)

service.interceptors.request.use(
  config => {
    const method = config.method.toUpperCase()
    if (
      config.showProcessMessage !== false &&
      (method === 'POST' ||
        method === 'PUT' ||
        method === 'DELETE' ||
        typeof config.processMessage !== 'undefined')
    ) {
      config.$$dialog = Message({
        message: (config.processMessage || '正在执行操作...'),
        type: 'info',
        duration: 0
      })
    }
    if (config.ignoreInterceptors === true) return config
    return config
  },
  error => {
    if (error.config.$$dialog) error.config.$$dialog.close()
    return Promise.error(error)
  }
)
/**
  * 请求失败后的错误统一处理
  * @param {Number} status 请求失败的状态码
  */
const errorHandle = (status, other) => {
  let html = ''

  if (typeof other.errors !== 'undefined') {
    const data = other.errors
    for (const key in data) {
      if (data.hasOwnProperty(key)) {
        const element = data[key]
        html += element + '<br/>'
      }
    }
    MessageBox.alert(html, '系统提示', {
      dangerouslyUseHTMLString: true,
      type: 'error'
    })
    // 如果不存在errors 存在message
  } else if (typeof other.message !== 'undefined' && other.message !== null) {
    if (typeof other.message === 'object') {
      html = ''
      for (const key in other.message) {
        if (other.message.hasOwnProperty(key)) {
          const element = other.message[key]
          html += element + '<br/>'
        }
      }
      tip(html)
    } else {
      tip(other.message)
    }
  }
  if (status === 401) {
    // location.href = '/login?redirect=' + location.pathname + location.search
  }
}

// response interceptor
service.interceptors.response.use(
  response => {
    if (response.config.$$dialog) response.config.$$dialog.close()
    if (response.config.ignoreInterceptors === true) return response

    if (response.status === 204 || !response.data) {
      return {
        status_code: 200,
        message: '操作成功！'
      }
    } else if (response.status === 201) {
      const data = response.data
      data.status_code = 200
      return data
    } else {
      return response.data
    }
  },

  error => {
    if (error.config.$$dialog) error.config.$$dialog.close()
    const { response } = error
    if (response) {
      // 请求已发出，但是不在2xx的范围
      errorHandle(response.status, response.data)
      return Promise.reject(response)
    } else {
      store.commit('changeNetwork', false)
    }
  })
export default service
